﻿HANDLE hPort = 0;
char   DioBuf[1]; // 8-bit DIO (can be more)
                     //    bit 0~3: DIs
                     //    bit 4~7: DOs
                  
WORD   AioBuf[6]; // 16-bit x6 AIO buffers (can be more)

// The Modbus RTU Slave API shares the same address for DI and DO.
// So, each bit can work as DI or DO depending on READ or WRITE.
// For DO, user have to read/write the bit in TouchPAD.
// For DI, user have to read it only in TouchPAD.
// The API does not care about its DI or DO, even its purpose.
// User have to define all these depending on his application.

// The Modbus RTU Slave API shares the same address for AI and AO too.

// In slave mode, all DI/DO satus are all off when TouchPAD rebooted.
// Thus, we need to use a bit as flag to tell the Master to initalize
// the DIO status of the TouchPAD first.

void Frame12OnCreate()
{
  if ( ! hPort )  // port is not opened?
  {
      hPort = uart_Open("COM1,9600,N,8,1");  // open port

      memset(DioBuf, 0, sizeof(DioBuf));  // Clear buffer
      memset(AioBuf, 0, sizeof(AioBuf));

      // NetID, DIO start address, channels, char buffers,
      //        AIO start address, channels, word buffers
      mrs_RegisterSlave(1, 0, 8, DioBuf, 0, 6, AioBuf); // Supports one slave only.
  }
}

void Timer4OnExecute(tWidget *pWidget)
{
  unsigned long mrs_status;

  if ( hPort )  // port is opened?
  {
     mrs_status = mrs_ProcessCmd(hPort); // retrieve Modbus RTU command and process it
     if ( mrs_status & MRS_DIO_DIRTY )  // DIO is updated
     {
         // Bit 4 ~ 7 as DO
        CheckBoxSelectedSet(&CheckBox5, DioBuf[0] & (1 << 4));
        CheckBoxSelectedSet(&CheckBox7, DioBuf[0] & (1 << 5));
        CheckBoxSelectedSet(&CheckBox8, DioBuf[0] & (1 << 6));
        CheckBoxSelectedSet(&CheckBox9, DioBuf[0] & (1 << 7));
     }
  }
}

// Set Bit 4 ~ 7
void CheckBox5OnChange(tWidget *pWidget, unsigned long bSelected)
{
    // Use CheckBoxTagGet() to read user predefined tag value.
    // The tag value indicates bit-index base on 0.
    if ( bSelected )
       DioBuf[0] |= (1 << CheckBoxTagGet(pWidget));
    else
       DioBuf[0] &= ~ (1 << CheckBoxTagGet(pWidget));
}