﻿
// The simple TCP Echo Server supports multiple client connections.
// Callback functions are required.

#define PORT         23
#define TIMEOUT      500
#define BUF_LEN      64

static tHandle h = INVALID_HANDLE;  // TCP Server handle (-1 == INVALID_HANDLE)

static char szConnectionState[30];
static char szOperationState[20];

// This is the callback function when data received.
void UserReceived(tHandle hnd)
{
   static unsigned char recv_buf[BUF_LEN];
   int ret = 0;
	 
   // Reads data through a TCP session.
   ret = hmi_TCPReadEx(hnd, recv_buf, BUF_LEN, TIMEOUT);
	LabelTextSet(&Label12, (char *)recv_buf);
	if (ret > 0)
	{
		// Write back (ECHO) to the TCP session immediately (no waiting in the queue).
		hmi_TCPOutput(hnd, recv_buf, ret);
	}
}

// This is the callback function when session disconnected.
void UserDisconnected(tHandle hnd)
{
	usprintf(szConnectionState, "#%d] off", hnd);
   LabelTextSet(&Label10, szConnectionState);
}

// This is the callback function when incoming connection is accepted.
void UserAccepted(tHandle hnd)
{
	char szIP[20];
	unsigned long ulIP = hmi_TCPGetRemoteIP(hnd);
	unsigned short usPort = hmi_TCPGetRemotePort(hnd);
	hmi_IPToStr(ulIP, szIP);
	
	usprintf(szConnectionState, "#%d] %s:%d", hnd, szIP, usPort);
   LabelTextSet(&Label10, szConnectionState);

   // Set callback functions for the new connection (child session).
	hmi_TCPSetCallbackDisconnected(hnd, UserDisconnected);	
	hmi_TCPSetCallbackReceived(hnd, UserReceived);	
}

void BitButton15OnClick(tWidget *pWidget)   // Start
{
	if ( h > INVALID_HANDLE ) return; // already have connection, so skip.
	
   // Allocate a TCP session if possible
	// The return value of the function is new handle value (0 to 31), or error code (-1 to -2).
   h = hmi_TCPNew();  

   if ( h > INVALID_HANDLE )  // new session allocated, OK.
   {
		LabelTextSet(&Label4, "Listening");
		hmi_TCPListen(h, PORT);
		
      // Set callback function for the server.
		hmi_TCPSetCallbackAccepted(h, UserAccepted);

      TimerEnabledSet(&Timer9, 1); // allow LED flash
   }
   else // Failure
   {
	  usprintf(szOperationState, "Err= %d", h);
	  LabelTextSet(&Label4, szOperationState);
	  
	  h = INVALID_HANDLE; // don't keep the error code in handle
   }
}

void BitButton6OnClick(tWidget *pWidget)      // Stop
{
  if ( h > INVALID_HANDLE ) 
  {
     hmi_TCPClose(h);  //closes the server and child sessions.
     h = INVALID_HANDLE;
     LabelTextSet(&Label4, "OFF");
  }
}

void Timer5OnExecute(tWidget *pWidget)
{
  unsigned long ulIP = hmi_TCPGetLocalIP();
  if ( (ulIP == 0) || (ulIP == 0xFFFFFFFF) ) return; // DHCP not ready
  
  usprintf(szOperationState, "Click Start");
  LabelTextSet(&Label4, szOperationState);
  
  BitButtonEnabledSet(&BitButton15, 1); // allow "Start"
  TimerEnabledSet(&Timer5, 0); // no more checking on "IP"
}

void Timer9OnExecute(tWidget *pWidget)
{
   static int iLED = 0;

   iLED++;
   iLED %= 2;

   hmi_SetLED(iLED);
}